const pool = require('./src/connect.js').connect
const mix = require('./src/mix.js')
const util = require('util')

const timeParser = require('easy-time-parser')
const cipher = require('my-easy-crypto')

//判断增删改操作结果
const checkAffected = (rows) => {
	if (!rows || !rows.affectedRows) {
		return {
			msg: 'did not worked',
			affected: false,
			num: 0
		}
	} else if (rows.affectedRows == 0) {
		return {
			msg: 'nothing changed',
			affected: false,
			num: rows.affectedRows
		}
		//TODO
	} else if (rows.affectedRows == 1) {
		return {
			msg: 'yes,1 row changed',
			affected: true,
			num: rows.affectedRows
		}
	} else {
		return {
			msg: `yes,${rows.affectedRows} changed`,
			affected: true,
			num: rows.affectedRows
		}
	}
}

// 判断查询结果
const checkSelected = (rows) => {
	if (!rows || rows.length == 0) {
		return {
			msg: '查询结果不存在',
			success: false,
			result: [],
			count: 0
		}
	} else {
		return {
			msg: '查询结果正常',
			success: true,
			result: rows
		}
	}
}

class database extends mix(timeParser,cipher) {
	constructor(conf) {
		super()
		this._pool = pool(conf)
	}
	_query(_pool, sql, params, cb) {
		//如果条件确定 则sql语句的替换参数可以为空
		let _params = params || null
		//创建一个连接池
		_pool.getConnection((err, conn) => {
			if (err) {
				//连接错误
				cb(err, null)
			} else {
				//防止异步操作某一段发生err 从而可以rollback()
				//只能回滚一条数据 两段式数据如何回滚 TODO
				conn.beginTransaction((err) => {
					if (err) {
						cb(err)
					} else {
						conn.query(sql, _params, (err, rows) => {
							//语句执行错误 回滚					
							if (err) {
								cb(err, null)
								return conn.rollback(err => {
									console.log('rollback err', err)
								})
							} else {
								//提交运行
								conn.commit(err => {

									if (err) {
										console.log('commit err \n', err)
										cb(err, null)
										return conn.rollback(err => {
											console.log('rollback err', err)
										})
									} else {
										//commit成功
										cb(null, rows)
										conn.release() //放回连接池
									}
								})
							}
						})
					}
				})
			}
		})
	}

	async async_query(sql, params) {
		let query = util.promisify(this._query)
		let back = await query(this._pool, sql, params)
		return back
	}

	showPool() {
		if (this._pool) {
			console.log(this._pool)
		} else {
			console.log('no pool')
		}
	}

	endPool() {
		if (this._pool) {
			this._pool.end()
			console.log('-------连接池已关闭------')
		} else {
			console.log('--------无连接池------')
		}
	}

	//返回此表数据条数
	async count(tableName, cb) {
		if(!tableName){
			throw new Error('MUST GIVE A TABLENAME')
		}
		let _sql = 'select count(*) as count from ??'
		let back = await this.async_query(_sql, tableName)
		let end = checkSelected(back)
		return end
	}
	//插入数据
	async insert(tableName, obj, cb) {
		let _sql = "insert into ?? set ?"
		let back = await this.async_query(_sql, [tableName, obj])
		let end = checkAffected(back)
		return end
	}

	//删除数据 只有一个过滤条件
	//arr = [tableName,obj]
	async delete(tableName, filter) {
		if (!tableName) {
			throw new Error('MUST GIVE A TABLE NAME')
		}
		if (!filter) {
			throw new Error('MUST HAVE A FILTER,IF YOU WANT DELETE ALL ,PLEASE USE THE DROP METHOD')
		}
		let _sql = 'delete from ?? where ?'
		let back = await this.async_query(_sql, [tableName, filter])
		let end = checkAffected(back)
		return end
	}

	//更新只有一个where条件的数据 
	//arr = [string,obj,obj]
	async update(tableName, val, filter) {
		if (!tableName) {
			throw new Error('MUST GIVE A TABLE NAME')
		}
		if (!val) {
			throw new Error('MUST GIVE ALL KEY-VALUE(OBJECT) YOU WANT TO UPDATE')
		}
		if (!filter) {
			throw new Error('MUST GIVE A FILTER,IF U WANT UPDATE ALL ,PLEASE USE UPDATEALL METHOD')
		}

		let _sql = 'update ?? set ? where ?'
		let back = await this.async_query(_sql, [tableName, val, filter])
		let end = checkAffected(back)
		return end
	}

	//更新所有
	async updateAll(tableName, val) {
		if (!tableName) {
			throw new Error('MUST GIVE A TABLE NAME')
		}
		if (!val) {
			throw new Error('MUST GIVE ALL KEY-VALUE(OBJECT) YOU WANT TO UPDATE')
		}

		let _sql = `update ?? set ? `
		let back = await this.async_query(_sql, [tableName, val])
		let end = checkAffected(back)
		return end
	}

	//执行完整的sql 或者同时传递参数
	async execute(sql, arr) {
		let _sql = sql
		// 如果没有变量参数 则自动传arr=[]
		let _params = arr || null
		let back = await this.async_query(_sql, _params)
		let end = checkSelected(back)
		return end
	}

	async hasVal(tableName,col,val){
		if(!tableName||!col||!val){
			throw new Error('PLEASE GIVE FULL PARAMS')
		}
		let _sql = `select * from ${tableName} where ${col} = ${val}`
		let back = await this.execute(_sql)
		return back.success		
	}
	//创建数据库
	async createDB(database) {
		let _sql = `create database if not exists ${database} `
		let back = await this.async_query(_sql)
		//let end = checkAffected(back)
		return back
	}

}

module.exports = database